@@ -19,7 +19,6 @@
 #include "cap.h"
 #include "quota.h"
 #include "fs.h"
-#include "tmpl.h"
 #include "dist.h"
 
 typedef struct {
@@ -73,13 +72,15 @@ struct log_s {
         int enable;
 };
 
-typedef struct {
+struct vps_param {
 	struct log_s log;
 	vps_res res;
 	vps_res del_res;
 	vps_opt opt;
 	struct mod_action *mod;
-} vps_param;
+	struct vps_param *g_param;
+};
+typedef struct vps_param vps_param;
 
 /** Setting VPS resources.
  *
