@@ -826,6 +826,8 @@ void merge_conf(struct Cveinfo *ve, vps_res *res)
 		ve->hostname = strdup(res->misc.hostname);
 	if (res->fs.root != NULL)
 		ve->ve_root = strdup(res->fs.root);
+	if (res->fs.private != NULL)
+		ve->ve_private = strdup(res->fs.private);
 }
 
 int read_ves_param()
@@ -834,6 +836,7 @@ int read_ves_param()
 	char buf[128];
 	vps_param *param;
 	char *ve_root = NULL;
+	char *ve_private = NULL;
 	int veid;
 
 	param = init_vps_param();
@@ -841,15 +844,19 @@ int read_ves_param()
 	vps_parse_config(0, CFG_FILE, param, NULL);
 	if (param->res.fs.root != NULL)
 		ve_root = strdup(param->res.fs.root);
+	if (param->res.fs.private != NULL)
+		ve_private = strdup(param->res.fs.private);
 	free_vps_param(param);
 	for (i = 0; i < n_veinfo; i++) { 
 		veid = veinfo[i].veid;
 		param = init_vps_param();
 		snprintf(buf, sizeof(buf), SCRIPT_DIR "%d.conf", veid);
 		vps_parse_config(veid, buf, param, NULL);
+		merge_conf(&veinfo[i], &param->res);
 		if (veinfo[i].ve_root == NULL)
 			veinfo[i].ve_root = strdup(ve_root);
-		merge_conf(&veinfo[i], &param->res);
+		if (veinfo[i].ve_private == NULL)
+			veinfo[i].ve_private = strdup(ve_private);
 		free_vps_param(param);
 	}
 	if (ve_root != NULL)
@@ -1280,6 +1287,12 @@ int get_mounted_status()
 			continue;
 		if (veinfo[i].status == VE_RUNNING)
 			continue;
+		if (veinfo[i].ve_private == NULL ||
+			!stat_file(veinfo[i].ve_private))
+		{
+			veinfo[i].hide = 1;
+			continue;
+		}
 		if (vps_is_mounted(veinfo[i].ve_root))
 			veinfo[i].status = VE_MOUNTED;
 	}
